"""Constants used by the metarial light and dark theme"""
BLACK = "#000000"
WHITE = "#FFFFFF"
DARK_PRIMARY_TEXT = "rgba(0,0,0, 0.87)"
DARK_SECONDARY_TEXT = "rgba(0,0,0, 0.54)"
DARK_DISABLED_TEXT = "rgba(0,0,0, 0.38)"
DARK_DIVIDERS = "rgba(0,0,0, 0.12)"
DARK_FOCUSED = "rgba(0,0,0, 0.12)"
LIGHT_PRIMARY_TEXT = WHITE
LIGHT_SECONDARY_TEXT = "rgba(255,255,255, 0.7)"
LIGHT_DISABLED_TEXT = "rgba(255,255,255, 0.5)"
LIGHT_DIVIDERS = "rgba(255,255,255, 0.12)"
LIGHT_FOCUSED = "rgba(255,255,255, 0.12)"

PRIMARY_TEXTS = {
    "Light": LIGHT_PRIMARY_TEXT,
    "Dark": DARK_PRIMARY_TEXT,
}
